#pragma once

#include <uvm>
#include <crave2uvm.h>
#include "ubus_transfer.h"

template<typename REQ = ubus_transfer, typename RSP = REQ>
class ubus_base_sequence : public uvm_randomized_sequence<REQ, RSP> {
public:
    UVM_OBJECT_PARAM_UTILS(ubus_base_sequence<REQ, RSP>);

    ubus_base_sequence(crave::crv_object_name name = "ubus_base_seq") : uvm_randomized_sequence<REQ, RSP>(name) {
        //Not in UVM SystemC? => set_automatic_phase_objection(1);
    };
};

template<typename REQ = ubus_transfer, typename RSP = REQ>
class write_double_word_seq : public ubus_base_sequence<REQ, RSP> {
public:
    crv_variable<sc_bv<16> > start_addr;
    crv_variable<sc_bv<8> > data0;
    crv_variable<sc_bv<8> > data1;
    crv_variable<sc_bv<8> > data2;
    crv_variable<sc_bv<8> > data3;
    crv_variable<sc_bv<8> > data4;
    crv_variable<sc_bv<8> > data5;
    crv_variable<sc_bv<8> > data6;
    crv_variable<sc_bv<8> > data7;
    crv_variable<unsigned int> transmit_del;

    crv_constraint transmit_del_ct{transmit_del() <= 10};

    UVM_OBJECT_UTILS(write_double_word_seq);

    write_double_word_seq(crave::crv_object_name name = "write_double_word_seq") : ubus_base_sequence<REQ, RSP>(name) {
    };

    void body() {
        REQ* req = new REQ();
        UVM_DO_WITH(req, (req.addr() == start_addr(),
                req.read_write() == ubus_read_write_enum::WRITE,
                req.size() == 8,
                req.data()[0] == data0(), req.data()[1] == data1(),
                req.data()[2] == data2(), req.data()[3] == data3(),
                req.data()[4] == data4(), req.data()[5] == data5(),
                req.data()[6] == data6(), req.data()[7] == data7(),
                req.error_pos() == 1000,
                req.transmit_delay() == transmit_del()));
        std::ostringstream stringStream;
        stringStream << "Writing " << this->get_sequence_path() << ": ";
        stringStream << "addr = '" << req.addr << ", ";
        stringStream << "data[0] = '" << req.data[0] << ", ";
        stringStream << "data[1] = '" << req.data[1] << ", ";
        stringStream << "data[2] = '" << req.data[2] << ", ";
        stringStream << "data[3] = '" << req.data[3] << ", ";
        stringStream << "data[4] = '" << req.data[4] << ", ";
        stringStream << "data[5] = '" << req.data[5] << ", ";
        stringStream << "data[6] = '" << req.data[6] << ", ";
        stringStream << "data[7] = '" << req.data[7] << std::endl;
        //TODO: Why is UVM_HIGH is not declared here?
        UVM_INFO(get_type_name(), stringStream.str(), 300);
    }
};
