#pragma once

#include <uvm>
#include <systemc.h>
#include <crave2uvm.h>
#include <crave/ConstrainedRandom.hpp>
#include <crave/experimental/Experimental.hpp>

using crave::crv_variable;
using crave::crv_vector;
using crave::crv_constraint;
using crave::inside;
using std::string;

CRAVE_BETTER_ENUM(ubus_read_write_enum, NOP = 0, READ = 1, WRITE = 2);

class ubus_transfer : public uvm_randomized_sequence_item {
public:
    crv_variable<ubus_read_write_enum> read_write;
    crv_variable<sc_bv<16> > addr;
    crv_variable<unsigned int> size;
    crv_vector<sc_bv<8> > data;
    crv_vector<sc_bv<4> > wait_state;
    crv_variable<unsigned int> error_pos;
    crv_variable<unsigned int> transmit_delay;
    string master = "";
    string slave = "";

    crv_constraint c_read_write{inside(read_write(), std::set<ubus_read_write_enum> {
            ubus_read_write_enum::READ, ubus_read_write_enum::WRITE
        })};

    crv_constraint c_size{inside(size(), std::set<int> {
            1, 2, 4, 8
        })};

    crv_constraint c_data_wait_size{
        data().size() == size(),
        wait_state().size() == size()
    };

    crv_constraint c_transmit_delay{transmit_delay() <= 10};

    UVM_OBJECT_UTILS(ubus_transfer);

    ubus_transfer(string name = "ubus_transfer_inst") : uvm_randomized_sequence_item(name) {
    };
};